# encoding: UTF-8
=begin
Copyright 2014 TIG (c)  
All rights reserved.  
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,  
INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND  
FITNESS FOR A PARTICULAR PURPOSE.  

Usage:  
Plugins menu  "TIG.Save_All_Components"
If the model has never been saved then nothing happens !  
Otherwise all components in the model are saved into a folder made with the  
model named, 'ModelName_Components'  
If a component is nested and not available as a stand-alone component then it is  
put into a subfolder named after its parent component's definition: if it is  
used in several components it is duplicated in appropriate subfolders.  
If a component's name contains unsuitable characters for a saved SKP name  
[e.g. / or :], then those characters are substituted [with "_"],  
if a component name is already used it is incremented, so #0 >>> #1 etc.  
Progress/success/failure is reported in the Ruby Console.  
On completion the 'ModelName_Components' folder is opened.  
In v2 saving nested components is optional Y|N

Version:  
1.0	20140625 First issue.  
1.1 20140626 Illict character trapping improved.
2.0 20211005 Option to save nested components. File open made newer MAC compliant.

=end

module TIG
 module Save_All_Components
	###menu
	unless @loaded
		UI.menu("Plugins").add_item("TIG.Save_All_Components"){ self.run() }
	end
	@loaded=true
	###
  def self.run()
	  m=Sketchup.active_model
	  p=m.path.tr("\\","/")
	  if p.empty? # i.e. NOT Untitled/unsaved
      UI.messagebox("TIG.Save_All_Components:\n\nSave this new model first !")
    else
      f=File.join(File.dirname(p), "#{m.title}_Components")
      begin
        Dir.mkdir(f)
      rescue
      end
      ns={}
      ### check if nested ?
      yn=UI.messagebox("TIG.Save_All_Components:\n\Put any nested components in subfolders ?", MB_YESNO)
      if yn==IDYES
        nested=true
      else
        nested=false
      end
      puts "nested = #{nested}"
      ###
      m.definitions.each{|d|
        next if d.group? || d.image?
        n=d.name.gsub(/[\<\>\/\\\:\?\"\%\\*|]/, '_').gsub(/^\./, '-')
        ###
        while ns[n]==true
          n.next!
        end
        if nested #ignore nested instances in main folder
          d.instances.each{|i|
            #p i.parent.name 
            next if i.parent.is_a?(Sketchup::ComponentInstance) && ! i.parent.is_a?(Sketchup::Group)
            s=File.join(f, i.parent.name.gsub(/[\<\>\/\\\:\?\"\%\*|]/, '_')).gsub(/^\./, '-')
            begin
              Dir.mkdir(s)
            rescue
            end
            c=File.join(s, n)
            begin
              d.save_as(c)
            if File.exist?(c+".skp")
              ns[n]=true
              puts "Saved #{c+".skp"}"
            else
              ns[n]=false
              puts "Failed to save #{c+".skp"}"
            end
            rescue
              ns[n]=false
              puts "Failed to save #{c+".skp"}"
            end
          }
        else # all defns into folder
          c=File.join(f, n)
          begin
            d.save_as(c)
            if File.exist?(c+".skp")
              ns[n]=true
              puts "Saved #{c+".skp"}"
            else
              ns[n]=false
              puts "Failed to save #{c+".skp"}"
            end
          rescue
            ns[n]=false
            puts "Failed to save #{c+".skp"}"
          end
        end  
      }
      puts
      ### MAC/PC fix v2
      if Sketchup.version.to_i >= 14 && Sketchup.platform == :platform_osx
        # >= v2014 MAC - use one of the many common standard Ruby methods...
        # all of these work
        `open "#{f}"`
        # or
        #%x[ open "#{f}" ]
        # or
        #system("open #{f.inspect}")
        # there are even more variants, this opens any file type in the Finder...
        #%x[ open -R #{f.inspect}]
      else # it is WIN or old MAC
        UI.openURL("file:///#{f}")
      end
    end#if
  end
 end
end
###